import time
import requests

class Utils():

  ## Setter
  # Set IP
  def use(self, newIP):
    self.local_ip = newIP
    print("Amsel CLI uses now %s for requests" % self.local_ip)

  # Set new default speed
  def setSpeed(self, newSpeed):
    print("Amsel CLI used %s as default speed now it uses %s") % (self.speed, newSpeed)
    self.speed = newSpeed

  ## Getter
  # Get the current default speed
  def getSpeed(self):
    return self.speed

  # Returns teh distance measured by the IR sensor
  def getDistance(self):
    endpoint = "/distance"
    response = self.get(endpoint)
    return response.text

  # Perform get request and return status
  def get(self, endpoint):
    path = "http://" + str(self.local_ip) + str(endpoint)
    response = requests.get(path)
    return response

  # Return IP
  def IP(self):
    print("Amsel CLI uses %s as IP" % self.local_ip)
    return self.local_ip

  # Return address
  def address(self):
    print("Amsel CLI uses %s as network address" % self.local_address)
    return self.local_address

  ## Helper
  # Set amsel to sleep for a certain amount of time
  def sleep(self, duration=0):
    time.sleep(duration)


import time
import random

class Skills():

  def __init__(self, speed):
    self.speed = speed

  # Define movement functions.for
  def forward(self, speed=0):
    if not speed: speed = self.speed
    endpoint = "/forward?speed=%s" % speed
    response = self.get(endpoint)
    return response.status_code

  def backward(self, speed=0):
    if not speed: speed = self.speed
    endpoint = "/reverse?speed=%s" % speed
    response = self.get(endpoint)
    return response.status_code

  def left(self, speed=0):
    if not speed: speed = self.speed
    endpoint = "/left?speed=%s" % speed
    response = self.get(endpoint)
    return response.status_code

  def right(self, speed=0):
    if not speed: speed = self.speed
    endpoint = "/right?speed=%s" % speed
    response = self.get(endpoint)
    return response.status_code

  # Stop movement
  def stop(self):
    endpoint = "/stop"
    response = self.get(endpoint)
    return response.status_code

  # Infinite run with obsticle avoidance
  def go(self, speed=100, duration=-1, proximity=30):
    start = time.time()
    now = time.time()

    try:
      while (now - start) < duration or duration == -1:
        self.forward(speed)
        distance = self.getDistance()
        if distance:
          if int(distance) < int(proximity):
            obsticle = True
          else:
            obsticle = False

          if obsticle:
            self.stop()
            randomDirection = random.randint(0, 1)
            randomDuration = random.random()*1
            if randomDirection:
              self.right()
            else:
              self.left()
            self.sleep(randomDuration)
            self.stop()
        else:
          print("Miss")
        self.sleep(0.1)
        now = time.time()
    except KeyboardInterrupt:
      self.stop()
      print("Wow what a run!")

  # Print a message to the display
  def print(self, text):
    endpoint = "/left?string=%s" % text
    response = self.get(endpoint)
    return response.status_code



class Connection():

  def __init__(self, local_ip):
    self.local_ip = local_ip
    self.local_address = "amsel.local"

  # Log into network
  async def login(self, ssid, passphrase):
    try:
        return await self.get("/login?ssid=" + ssid + "&pass=" + passphrase)
    except IOError:
        print("Something went wrong we could not connect you to the network!")
    
